@@ -20,7 +20,7 @@
 ** jwhitham@globalnet.co.uk
 */
 
-/* htget version 0.91 */
+/* htget version 0.92 */
 
 #include <stdio.h>
 #include <string.h>
@@ -48,7 +48,7 @@ const char base64_table[64]= "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 #define MY_NAME			"alternate 1"
 #define CONTENT_LENGTH		"Content-Length:"
 
-#define MAX_URLS		16
+#define MAX_URLS		32
 
 #define OPTIONS			15
 
@@ -104,6 +104,7 @@ int main ( int argc , char * argv [] )
 
 	/* open the resource file. */
 	/* first search cmd line params to see if one has been specified. */
+	ResourceFile = NULL ;
 	for ( I = 1 ; I < argc ; I ++ )
 	{
 		if ( strncasecmp ( "--rcfile=" , argv [ I ] , 9 ) == 0 )
@@ -131,17 +132,16 @@ int main ( int argc , char * argv [] )
 				strcat ( Temp , "/.htgetrc" ) ;
 			}
 			ResourceFile = fopen ( Temp , "rt" ) ;
-			if ( ResourceFile == NULL)
-			{ 
-				strcpy ( Temp, "/etc/htget.conf" );
-				ResourceFile = fopen ( Temp, "rt" );
-			}
 			if ( ResourceFile == NULL )
 			{
-				fprintf ( stderr , "%s: couldn't find a htget resource file in the current\n"
-						"directory or your home directory. Please read\n"
-						"the documentation that came with htget for more details\n" ,
-						argv [ 0 ] ) ;
+				ResourceFile = fopen ( "/etc/htget.conf" , "rt" ) ;
+				if ( ResourceFile == NULL )
+				{
+					fprintf ( stderr , "%s: couldn't find a htget resource file in the current\n"
+							"directory or your home directory. Please read\n"
+							"the documentation that came with htget for more details\n" ,
+							argv [ 0 ] ) ;
+				}
 			}
 		}
 	}
@@ -157,7 +157,7 @@ int main ( int argc , char * argv [] )
 	ShowPercentage = 1 ;
 	ShowBytesPerSecond = 1 ;
 	strcpy ( DownloadsDir , "./" ) ;
-	strcpy ( UserAgent , "linux htget 0.91" ) ;
+	strcpy ( UserAgent , "linux htget 0.92" ) ;
 	ShowBanner = 1 ;
 
 	if ( ResourceFile != NULL )
@@ -242,7 +242,7 @@ int main ( int argc , char * argv [] )
 
 	if ( ShowBanner )
 	{
-		fprintf ( stderr , "HTGET (c) J Whitham 1998  <jwhitham@globalnet.co.uk> version 0.91\n" 
+		fprintf ( stderr , "HTGET (c) J Whitham 1998  <jwhitham@globalnet.co.uk> version 0.92\n" 
 			"See README. This program comes with NO WARRANTY of any kind.\n\n" ) ;
 	}
 
@@ -321,7 +321,7 @@ int main ( int argc , char * argv [] )
 			{
 				strcpy ( RetrieveURL [ URLs ] , argv [ I ] ) ;
 				URLs ++ ; 
-				if ( URLs >= MAX_URLS )
+				if ( URLs > MAX_URLS )
 				{
 					fprintf ( stderr , "The maximum number of URLs that can be specified at the command line is %d.\n" , MAX_URLS ) ;
 					return ( 0 ) ;
@@ -426,14 +426,14 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 	long		CurrentSize ;
 	long		RangeSize = 0 , RangeStart = 0 ;
 	/* authenticate */
-	char		* UserPass ;
+	char		UserPass [ MAXLEN ] ;
 	struct stat	ConnectionStatistics ;
+	char		* base64str ;
 	/* timing */
 	time_t		StartTime , CurrentTime , TimeoutTime ;
 	time_t		RedundantReportsTimer ;
 
 
-	UserPass = (char *) malloc ( strlen ( USER_PASS ) + 1 ) ;
 	strcpy ( UserPass , USER_PASS ) ;
 	
 	if ( ! ProcessURL ( RetrieveURL , Hostname , Filename , ActualFilename , & Port ) )
@@ -456,7 +456,7 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 	strcpy ( recv_buf , ActualFilename ) ;
 	strcpy ( ActualFilename , DownloadsDir ) ;
 	strcat ( ActualFilename , recv_buf ) ;
-	
+
 	CurrentSize = 0 ;
 	/* read/resume check */
 	File = fopen ( ActualFilename , "rb" ) ;
@@ -486,12 +486,14 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 		}
 	
 		fprintf ( stderr , "Getting header for %s.." , RetrieveURL ) ;
+		base64str = ToBase64 ( UserPass , strlen ( UserPass ) ) ;
 		sprintf ( Request , "HEAD %s HTTP/1.0\r\n"
 			"Host: %s\r\n"
 			"User-agent: %s\r\n"
 			"Authorization: Basic %s\r\n\r\n" ,
-			    Filename , Hostname , UserAgent ,
-			    ToBase64 ( UserPass , strlen ( UserPass ) ) ) ;
+			    Filename , Hostname , UserAgent , base64str ) ;
+		free ( base64str ) ;
+
 		if ( Verbose )
 		{
 			fprintf ( stderr , "\n** Sent: %s" , Request ) ;
@@ -536,7 +538,6 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 		/* Avoid bugs in the next bit */
 	}
 		
-
 	socket = TCPConnect ( Hostname , Port ) ;
 	if ( socket == 0 )
 	{
@@ -545,21 +546,23 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 	fprintf ( stderr , "Getting data for %s.." , RetrieveURL ) ;
 	if ( Resuming )
 	{
+		base64str = ToBase64 ( UserPass , strlen ( UserPass ) ) ;
 		sprintf ( Request , "GET %s HTTP/1.1\r\n"
 			"Host: %s\r\n"
 			"User-agent: %s\r\n" 
 			"Range: bytes=%ld-%ld\r\n"
 			"Authorization: Basic %s\r\n\r\n" ,
 				Filename , Hostname , UserAgent ,
-				CurrentSize , ActualSize ,
-				ToBase64 ( UserPass , strlen ( UserPass ) ) ) ;
+				CurrentSize , ActualSize , base64str ) ;
+		free ( base64str ) ;
 	} else {
+		base64str = ToBase64 ( UserPass , strlen ( UserPass ) ) ;
 		sprintf ( Request , "GET %s HTTP/1.0\r\n"
 			"Host: %s\r\n"
 			"User-agent: %s\r\n" 
 			"Authorization: Basic %s\r\n\r\n" ,
-				Filename , Hostname , UserAgent ,
-				ToBase64 ( UserPass , strlen ( UserPass ) ) ) ;
+				Filename , Hostname , UserAgent , base64str ) ;
+		free ( base64str ) ;
 	}
 	if ( Verbose )
 	{
@@ -609,7 +612,7 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 		return ( ERROR_RESUME_IMPOSSIBLE ) ;
 	}
 
-	if ( CurrentSize > 0 )
+	if (( CurrentSize > 0 ) && ( Resuming ) )
 	{
 		File = fopen ( ActualFilename , "ab" ) ;
 	} else {
@@ -693,31 +696,6 @@ int HTGet ( char * RetrieveURL , int AllowResume )
 }
 
 
-/* MakeHTTPRequest
- * Makes a HTTP Request header for Hostname/Filename 
- * [ZB] */
-char * MakeHTTPRequest ( char * Hostname , char * Filename )
-{
-	char	tmp [ MAXLEN ] ;
-	char	*request ;
-	char	*pwstring ;
-
-	/* size of the two strings, plus ':' and trailing null */
-
-	pwstring = (char *) malloc ( strlen ( USER_PASS ) + 1 ) ;
-
-	strcpy ( pwstring , USER_PASS ) ;
-
-	sprintf(tmp, "GET %s HTTP/1.0\r\n"
-		"Host: %s\r\n"
-		"User-agent: snarf %s\r\n"
-		"Authorization: Basic %s\r\n\r\n",
-		Filename , Hostname , MY_NAME ,
-		ToBase64 ( pwstring , strlen ( pwstring ) ) ) ;
-	request = (char *) malloc ( sizeof ( tmp ) + 1 ) ;
-	strncpy ( request , tmp , MAXLEN ) ;
-	return ( request ) ;
-}
 
 /* TCPConnect
  * Gets a socket number for the host after finding it's address
@@ -774,6 +752,10 @@ int ReadLine ( int Socket , char * ReceiveBuffer )
 		{
 			break ;
 		}
+		if ( I > ( BIG_BUFFER_SIZE - 4 ))
+		{
+			break ;
+		}
 		rc = read ( Socket , & ch , 1 ) ;
 	}
 	if ( rc == 0 )
@@ -801,6 +783,12 @@ char * ToBase64(unsigned char *bin, int len)
 {
 	char *buf= (char *)malloc((len+2)/3*4+1);
 	int i=0, j=0;
+
+	if ( buf == NULL )
+	{
+		puts ( "ToBase64: error allocating memory for output" ) ;
+		exit ( 1 ) ;
+	}
 	
 	while(j<len-2){
 		buf[i++]=base64_table[bin[j]>>2];
@@ -1013,7 +1001,7 @@ int GetHeaderForRequest ( unsigned socket , char * request , char * Header )
 int NoCaseStrNCmp ( char * String1 , char * String2 , int Length )
 {
 	int	I ;
-	
+
 	for ( I = 0 ; I < Length ; I ++ )
 	{
 		if ( toupper ( String1 [ I ] ) != toupper ( String2 [ I ] ))
@@ -1024,7 +1012,7 @@ int NoCaseStrNCmp ( char * String1 , char * String2 , int Length )
 	return ( 1 ) ;
 }
 
-int GetHeaderData ( char * Header , char * FieldName , char * Record )
+int GetHeaderData ( char * Header , char FieldName [] , char * Record )
 {
 	int	I , J ;
 
@@ -1033,14 +1021,19 @@ int GetHeaderData ( char * Header , char * FieldName , char * Record )
 		if ( NoCaseStrNCmp ( & Header [ I ] , FieldName , strlen ( FieldName ) ) )
 		{
 			I += strlen ( FieldName ) ;
-			strcpy ( Record , & Header [ I ] ) ;
-			for ( J = 0 ; J < strlen ( Record ) ; J ++ )
+			if ( ( I + 2 ) < strlen ( Header ))
 			{
-				if ( Record [ J ] == '\n' )
+				strcpy ( Record , & Header [ I ] ) ;
+				for ( J = 0 ; J < strlen ( Record ) ; J ++ )
 				{
-					Record [ J ] = '\0' ;
-					break ;
+					if ( Record [ J ] == '\n' )
+					{
+						Record [ J ] = '\0' ;
+						break ;
+					}
 				}
+			} else {
+				return ( 0 ) ;
 			}
 			return ( 1 ) ;
 		}
